/*
 *
 * Description: Measure time of pulses on GP3.
 *				Use timer 1 and GPIO change interrupt.
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Copyright (C) 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f675.h>

__CONFIG(FOSC_INTRCIO & WDTE_OFF & BOREN_OFF & CP_OFF & CPD_OFF & MCLRE_OFF);

__EEPROM_DATA('T','u','t','o','-','1',0,0);

volatile bit getp = 0;
volatile unsigned int pulse = 0;

/* *************************************************************************************************** */

main (void)
{
	OPTION_REG = 0b10011000;					// set OPTION register
	INTCON = 0;									// disable all interrupts and clear all flags
	T1CON = 0;									// TMR1 CONTROL register
	ADCON0 = ANSEL = 0;							// turn off analog conversion
	CMCON = 0x07; 								// turn off analog comparator
	TRISIO = 0b001000;							// GP3 as input, all other as outputs
	GPIO = 0;									// all ports off
	TMR1ON = 1;									// TMR1 is now counting each usec elapsed.

/* *************************************************************************************************** */

	GPIF = 0;									// clear interrupt flag
	GPIE = 1;									// Port Change Interrupt Enable bit
	IOC = 0b001000;								// enable interrupts on GP3 pin change
	ei();										// enable general interrupts

/* *************************************************************************************************** *
	From now, any change in the value of the GP3 pin will rise the interrupt routine.
	TMR1 is used to count the time of a positive pulse.
* *************************************************************************************************** */

	for (;;) {									// loop for ever
		if (!getp) {							// pulse received
			if (pulse>=900 && pulse<=2100) {	// verify value
				if (pulse < 1200)				// low time pulse
					GPIO = 0b000001;			// set GP0
				else if (pulse > 1800)			// high time pulse
					GPIO = 0b000010;			// set GP1
				else							// medium time pulse
					GPIO = 0b100000;			// clear all outputs
			}
		}
	}
}

/* *************************************************************************************************** */

static void interrupt routine (void)
{
	if (GPIF) {									// GPIO port change
		GPIF = 0;								// clear interrupt flag
		if (GP3 && !getp) {						// start of pulse
			TMR1 = 1 - 128;						// reset TMR1 to number of cycles elapsed since entering the routine
			getp = 1;							// count pulse duration in progress
		}
		else if (!GP3 && getp) {				// end of pulse
			pulse = TMR1;						// get TMR1 value
			getp = 0;							// count pulse duration disabled
		}
	}
}

/* *************************************************************************************************** */
